/**
 * Glazed Lists 
 * Sorted JTables Performance Comparison
 * http://publicobject.com/glazedlists/
 *
 * COPYRIGHT 2004 O'DELL ENGINEERING LTD.
 */
import java.util.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import java.applet.*;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
// glazed lists
import ca.odell.glazedlists.*;
import ca.odell.glazedlists.swing.*;
// JDNC
import org.jdesktop.swing.*;
import javax.swing.table.*;

/**
 * @author <a href="mailto:jesse@odel.on.ca">Jesse Wilson</a>
 */
public abstract class AbstractBusySort implements Runnable {
    
    private static int[] modulos = new int[] { 100, 1000, 10000 };
    private int repeat = 1000;
    private int size = 1000;
    private int add = 1000;
    
    /**
     * Gets a table model that sorts the specified list.
     */
    public abstract JTable createSortedJTable(EventList unsorted);
    
    /**
     * Sets the number of repetitions, size, and number of elements to be added.
     */
    public void setParameters(int repeat, int size, int add) {
        this.repeat = repeat;
        this.size = size;
        this.add = add;
    }
    public void setParameters(String[] args) {
        if(args.length != 3) throw new RuntimeException("Usage BusySort <repeat> <size> <add>");
        this.repeat = Integer.parseInt(args[0]);
        this.size = Integer.parseInt(args[1]);
        this.add = Integer.parseInt(args[2]);
    }
    
    /**
     * Do the speed test.
     */
    public final void run() {
        // create the unsorted list
        BasicEventList unsorted = new BasicEventList();
        JTable sortedJTable = createSortedJTable(unsorted);
        
        // populate the list
        List initialData = new ArrayList();
        Random dice = new Random(0);
        for(int i = 0; i < size; i++) {
            initialData.add(new Integer(dice.nextInt(Integer.MAX_VALUE)));
        }
        unsorted.addAll(initialData);
        
        // start the clock
        long start = System.currentTimeMillis();
        
        // 1000 times, sort
        for(int j = 0; j < repeat; j++) {
            int sortColumn = j % modulos.length;
            clickOnColumnHeader(sortedJTable, sortColumn);

            for(int r = 0; r < sortedJTable.getRowCount(); r++) {
                Object cellValue = sortedJTable.getValueAt(r, sortColumn);
                //System.out.print(cellValue);
                //System.out.print(", ");
            }
            //System.out.println("");
        }
        
        // 1000 times, add
        for(int k = 0; k < add; k++) {
            unsorted.add(new Integer(dice.nextInt(Integer.MAX_VALUE)));
        
            for(int r = 0; r < sortedJTable.getRowCount(); r++) {
                sortedJTable.getValueAt(r, 0);
                //System.out.print(sortedTableModel.getValueAt(r, 0));
                //System.out.print(", ");
            }
            //System.out.println("");
        }
        
        // stop the clock
        long end = System.currentTimeMillis();

        // print results
        System.out.println("Runtime " + (end - start));
        System.exit(1);
    }
    
    private void clickOnColumnHeader(JTable table, int column) {
        Rectangle columnHeader = table.getTableHeader().getHeaderRect(column);
        MouseEvent mouseEvent = new MouseEvent(table.getTableHeader(), 0, 0, 0, columnHeader.x, columnHeader.y, 1, false, MouseEvent.BUTTON1);
        MouseListener[] listeners = table.getTableHeader().getMouseListeners();
        //System.out.println("mouse event " + mouseEvent);
        for(int i = 0; i < listeners.length; i++) {
            //System.out.println("Listener " + listeners[i] + " " + listeners[i].getClass());
            listeners[i].mouseClicked(mouseEvent);
        }
    }
    
    /**
     * A simple table format that returns three different modulos for three different
     * column values.
     */
    static class ModIntegerTableFormat implements TableFormat {
        
        public int getColumnCount() {
            return 3;
        }
        public String getColumnName(int column) {
            return "mod " + modulos[column];
        }
        public Object getColumnValue(Object baseObject, int column) {
            Integer asInt = (Integer)baseObject;
            return new Integer(asInt.intValue() % modulos[column]);
        }
    }
}
